#!/bin/bash
#----------------------------------------------------------------------
#
#  classmontage.sh  -  montages z-projections of class members
#
#  Version 1.00  10.01.2001 hw    last edit 09.05.2005
#
#----------------------------------------------------------------------

if [ $# -lt 2 -o $# -gt 6 ]; then
  echo "usage: `basename ${0}` #cls sffx [prj [seq [bx [bz]]]]" 1>&2
  exit 1
fi

# number of classes
nc=${1}

# file name suffix
sffx=${2}

# projection direction
prj=${3:-z}

# file with member sequence
seq=${4}

# box to determine density min/max of projected motifs
bx=${5}
bz=${6}


# main

if [ ${seq} ]; then
  if [ "x${seq}" = x- ]; then
    unset seq
  elif [ `cat ${seq} | wc -l` -ne ${nc} ]; then
    echo "`basename ${0}`: improper sequence file ${seq}" 1>&2
    exit 1
  fi
fi

set +e; let c=0; set -e

while [ ${c} -lt ${nc} ]; do
  cn=`printf "%03d-%03d%s" ${nc} ${c} ${sffx}`
  if [ x${prj} = x0 ]; then
    mnt="${mnt} ${cn}"
  else
    mnt="${mnt} ${cn}.prj"
    i3project -${prj} \
      ${cn} \
      ${cn}.prj
  fi
  set +e; let c=c+1; set -e

done

cmnt=`printf "%03d%s.tiff" ${nc} ${sffx}`

if [ ${seq} ]; then
  mnt=
  for s in `cat ${seq}`; do
    set +e; let c=10#${s}; set -e
    cn=`printf "%03d-%03d%s" ${nc} ${c} ${sffx}`
    if [ x${prj} = x0 ]; then
      mnt="${mnt} ${cn}"
    else
      mnt="${mnt} ${cn}.prj"
    fi
  done
  cmnt=`printf "%03d%s.dendr.tiff" ${nc} ${sffx}`
fi

cn=`printf "%03d-%03d%s" ${nc} 0 ${sffx}`
eval `i3stat -sh -s ${cn}.prj`

bxdef=`echo ${nx} | awk '{ print int(0.8*$0) }' `
bzdef=`echo ${ny} | awk '{ print int(0.8*$0) }' `

i3montage -sep 2 -col 10 \
  -ss ${bx:-${bxdef}} ${bz:-${bzdef}} -min -max -t uint8 -fmt tiff \
  ${mnt} \
  ${cmnt}

rm -f ${mnt}
