#!/bin/sh
#----------------------------------------------------------------------
#
#  dendrogram.sh  -  produces a dendrogram plot from cls file
#
#  Version 1.00  03.12.1998 hw    last edit 20.12.2000
#
#----------------------------------------------------------------------

set -e

if [ $# -lt 1 -o $# -gt 5 ]; then
  echo "usage: `basename ${0}` file [sx sy nc L|CH|CL]" 1>&2
  exit 1
fi

if [ ${1} = "-" ]; then
  :
else
  file=${1};
fi

awk \
  -v sx=${2:-1.0} \
  -v sy=${3:-1.0} \
  -v nc=${4:-0} \
  -v lb=${5:-N} '

  BEGIN {
    err=0;
    n=0; l=0;
    k=-1; k0=0;
    wmax=0;
    sx=sx+0.0; sy=sy+0.0;
    if ((sx <= 0) || (sy <= 0)) {
      printf("improper scale factors\n");
      err=1; exit;
    }
  }

  /^! eliminating/ {
    i=split($0,f);
    if (substr(f[i],1,1) == "(") { i--; }
    while (i > 2) {
      c1=f[i]+0;
      i1=n;
      while (--i1 >= 0) {
        if (cn[i1] == c1) { break; }
      }
      if (i1 >= 0) {
        printf("cannot eliminate cluster %d\n",c1);
        err=1; exit;
      }
      i--;
    }
  }

  /^! level/ {

    lv=substr($3,1,length($3)-1)+0;
    c1=$5+0; c2=$8+0;
    w=substr($11,3,length($11)-2);
    w=sqrt(w);
    if (w > wmax) { wmax=w; }
    m=substr($13,3,length($13))+0;

    if (lv != l++) {
      printf("incorrect sequence at level %d\n",lv);
      err=1; exit;
    }

    i1=n;
    while (--i1 >= 0) {
      if (cn[i1] == c1) { break; }
    }
    if (i1 < 0) {
      i1=n++;
      cn[i1]=c1;
      cl[i1]=-1;
      cr[i1]=-1;
      cc[i1]=0;
      cw[i1]=0;
      cs[i1]=1;
      co[i1]=0.5;
    }

    i2=n;
    while (--i2 >= 0) {
      if (cn[i2] == c2) { break; }
    }
    if (i2 < 0) {
      i2=n++;
      cn[i2]=c2;
      cl[i2]=-1;
      cr[i2]=-1;
      cc[i2]=0;
      cw[i2]=0;
      cs[i2]=1;
      co[i2]=0.5;
    }

    i=n++;
    cn[i]=c1;
    cl[i]=i1;
    cr[i]=i2;
    cc[i]=m;
    cw[i]=w;
    cs[i]=cs[i1]+cs[i2];
    co[i]=(co[i1]+cs[i1]+co[i2])/2;

  }

  /^! class/ {

    i=substr($3,1,length($3)-1)+0;
    if (++k != i) {
      printf("incorrect sequence at class %d [m=%d]\n",i,m);
      err=1; exit;
    }
    i=$5+0;
    cln[k0+k]=i;

  }

  /^!  #cls/ {

    if (++k != m) {
      printf("incorrect number of classes [m=%d]\n",m);
      err=1; exit;
    }
    cli[m]=k0;
    k0+=m;
    k=-1;

  }

  END {

    if (err) {
      printf("error exit\n");
      exit(err);
    }

    i=n-1;
    xmax=1.01*cs[i];
    wmax=1.05*wmax;
    bo=0.01;
    xs=xmax*(1+2*bo);
    ys=wmax*(1+2*bo);
    x0=bo/(1+2*bo);
    y0=bo/(1+2*bo);
    x1=(1+bo)/(1+2*bo);
    y1=(1+bo)/(1+2*bo);
    printf("#PLOT 2\n");
    if (sx >= sy) {
      printf("o\ns 0 0 %g %g\ne\n",sx*x1,sy*y1);
      printf("\\ 1 0  0 1 %g %g\n",sx*x0,sy*y0);
    } else {
      printf("o\ns 0 0 %g %g\ne\n",sy*y1,sx*x1);
      printf("\\ 0 1 -1 0 %g %g\n",sy*(y1-y0),sx*x0);
    }
    printf("FHelvetica\n");

    i=n-1;
    s=0;
    si[s]=i;
    s0[s]=0;
    sw[s]=wmax;
    fmt="m %g %g\nn %g %g\n";
    lbl="m %g %g\nTcb%d\nm %g %g\nTcb%d\n";
    chn="Tct%d\n";

    while (s >= 0) {

      i=si[s];
      x0=s0[s];
      y1=sw[s];
      s--;

      x1=x0+co[i];
      x2=x1; y2=cw[i];
      printf(fmt,x1/xs,y1/ys,x2/xs,y2/ys);
      i1=cl[i]; i2=cr[i];
      if ((i1 < 0) && (i2 < 0)) {
        continue;
      } else if ((i1 < 0) || (i2 < 0)) {
        printf("error exit\n"); exit(1);
      }
      if ((nc > 0) && (cc[i] >= nc)) {
        if (lb == "CH") {
          printf(chn,cn[i]);
        } else if (lb == "CL") {
          k=nc;
          k0=cli[k];
          while (--k >= 0) {
            if (cln[k0+k] == cn[i]) {
              break;
            }
          }
          printf(chn,k);
        }
        continue;
      }
      y1=y2;
      x1=x0+co[i1]; x2=x0+cs[i1]+co[i2];
      printf(fmt,x1/xs,y1/ys,x2/xs,y2/ys);
      if (lb == "L") {
        printf(lbl,x1/xs,y1/ys,cn[i1],x2/xs,y2/ys,cn[i2]);
      }

      s++;
      si[s]=i1;
      s0[s]=x0;
      sw[s]=y1;

      s++;
      si[s]=i2;
      s0[s]=x0+cs[i1];
      sw[s]=y2;

    }

    printf("x\n");

  }

' ${file}
