#!/bin/bash
#
# i3unstack.sh  version 2.0  Splits an image stack into separate files
#

if [ $# -lt 1 -o $# -gt 3 ]; then
  echo "usage: $( basename $0 ): file [prfx [sffx]]" 1>&2
  exit 1
fi

sffx=${3:-.img}

stat=1
eval $( i3stat -sh ${1}; echo stat=$? )
if [ ${stat} -ne 0 ]; then
  exit 1
fi
if [ ${dim} -ne 3 -a ${dim} -ne 4 ]; then
  echo "$( basename $0 ): improper dimension" 1>&2
  exit 1
fi

name=$( basename ${1} )

if [ ${2} ]; then
  name=${2}
else
  name=$( echo ${name} | awk '{ sub("[.].*$",""); print }' )
  name=${name:-noname}
fi

if [ ${dim} -eq 3 ]; then
  n=${nz}
  i=${oz}
  unset nz oz
else
  n=${ni}
  i=${oi}
fi

if [ ${n} -lt 100 ]; then
  fmt="%02d"
elif [ ${n} -lt 1000 ]; then
  fmt="%03d"
elif [ ${n} -lt 10000 ]; then
  fmt="%04d"
else
  fmt="%05d"
fi

j=0
while [ ${n} -gt 0 ]; do
  k=$( printf "${fmt}" ${j} )
  i3cut -s ${nx} ${ny} ${nz} 1 -o ${ox} ${oy} ${oz} ${i} -- ${1} ${name}${k}${sffx}
  set +e
  let i=i+1
  let j=j+1
  let n=n-1
  set -e
done
