#!/bin/bash
#
# i3alignmake.sh  version 2.3  create individual alignment scripts
#

[ $# -lt 2 ] && { echo "usage: $( basename $0 ) ref mode [params]" >&2; exit 1; }

set -e

# reference
ref=${1}
shift

# exec mode (nohup or submit.sh etc.)
mode=${1}
shift

# script directory
exe="i3align"


# parameters

unset cmr
if [ "${MRACMR}" ]; then
  cmr=" -cmr ${MRACMR}"
fi

unset msk
if [ "${MRAMSK}" ]; then
  msk=" ${MRAMSK}"
fi

unset wdgopt
if [ "w${WDGCOMP}" = wtrue -a ${DIM} = 3 ]; then
  wdgopt=" -wdg"
fi

if [ ${DIM} = 3 ]; then
  srch="-se"
else
  srch="-sr"
fi


# create shell scripts

mkdir ${exe}

echo -n >${exe}run.sh

cat ${SETS} |
while read map set; do
  set -e
  f="${exe}/mra-${set}"
  i3datalist ${I3D} ${SET}-raw |
  awk -v p=${set} '$1 == p' >${f}.trf
  if [ $( cat ${f}.trf | wc -l ) -gt 0 ]; then
    {
      echo '#!/bin/bash'
      echo 'set -e'
      echo "trf=${f}.trf"
      echo "i3d=${f}.i3d"
      echo "set=${SET}-${set}"
      echo "ref=${ref}"
      echo "ali=${f}-ali.trf"
      echo 'export I3FILEPATH="."$( while read dir map tlt; do echo -n ":${dir}"; done <'"${MAPS} )"
      echo 'echo start at $( date ) >&2'
      echo "cp ${I3D}" '${i3d}'
      echo "i3dataimport -dim ${DIM}"' ${i3d} ${set} ${trf}'
      echo "i3align -log${wdgopt} -cc ${MRACC:-xcf} -pkr ${MRAPKR}${cmr}${msk} ${srch} ${MRASRCH}" '${ref} ${i3d} ${set} ${set}-ali'
      echo 'i3datalist ${i3d} ${set}-ali >${ali}'
      echo 'rm -f ${i3d}'
      echo 'echo normal end at $( date ) >&2'
    } >${f}.sh
    chmod +x ${f}.sh
    case "${mode}" in
      nohup) echo "nohup ${f}.sh >${f}.log 2>&1" >>${exe}run.sh ;;
      cobra) echo "${I3COBRA:-custom.sh} ${f}.sh ${f}.log" >>${exe}run.sh ;;
      *)     echo "submit.sh ${f}.sh logfile=${f}.log $*" >>${exe}run.sh ;;
    esac
  else
    rm -f ${f}.trf
  fi
done
if [ "${mode}" = nohup ]; then
  if [ ${1} ]; then
    awk -v w=${1} '
      BEGIN { i = w }
      { print $0, "&"; i-- }
      w > 0 && !i { print "wait"; i = w }
      END { if ( w <= 0 || i < w ) { print "wait" } }
    ' ${exe}run.sh >${exe}runtmp.sh
    mv -f ${exe}runtmp.sh ${exe}run.sh
  fi
fi
