#!/bin/bash
#
# i3hacimagic.sh  version 2.1  wrapper script for Imagic
#

set -e

# msa stack file
msastack="${2}"

# msa set
msaset="${SET}-${1}"

# fraction of ignored high-variance outliers
hvo=${CLSHVO:-0.0}

# fraction of ignored high-variance class members
hvm=${CLSHVM:-0.0}

# convert to percentages
hvop=$( echo ${hvo} | awk '{ print 100 * $1 }' )
hvmp=$( echo ${hvm} | awk '{ print 100 * $1 }' )

# max number of active eigenimages
# (this can be a comma separated list, but Imagic only accepts a single number)
actei=$( echo ${CLSFACT} | awk '/^[0-9][0-9]*$/ { print; exit } { print "ERROR" }' )
if [ ${actei} = ERROR ]; then
  echo "invalid number of active eigenimages (CLSFACT)" >&2; exit 1
fi

# select number of classes for which to generate class averages
# (this can be a comma separated list, but Imagic only accepts a single number)
numcl=$( echo ${CLASSES} | awk '/^[0-9][0-9]*$/ { print; exit } { print "ERROR" }' )
if [ ${numcl} = ERROR ]; then
  echo "invalid number of classes" >&2; exit 1
fi

# following code adapted from Imagic accumulate file

export IMAGIC_BATCH=1

echo "! "
echo "! IMAGIC program: classify ---------------------------------------------"
echo "! "

${IMAGIC_ROOT}/msa/classify.e <<EOF
IMAGES/VOLUMES
${msastack}
${hvop}
${actei}
YES
${numcl}
${PRFX}-classes
EOF

echo "! "
echo "! END: classify ---------------------------------------------"
echo "! "

# convert to i3

vol=$( cat ${PRFX}-msa.trf | wc -l )

awk '{ for ( i = 1; i <= NF; i++ ) { print $i } } ' ${PRFX}-classes.cls |
awk '
  BEGIN { c = 0 }
  { stat = 1 }
  $0 != c + 1 { exit }
  {
    if ( getline <= 0 ) { exit }
    c = c + 1
    n = $0
    gsub( "[0-9]", "" )
    if ( $0 != "" ) { exit }
  }
  {
    if ( getline <= 0 ) { exit }
    if ( index( $0, "." ) == 0 ) { exit }
  }
  {
    for ( i = 1; i <= n; i++ ) {
      if ( getline <= 0 ) { exit }
      print $0, c
    }
    stat = 0
  }
  END {
    if ( stat ) { print "ERROR" }
    exit stat
  }
' |
sort -n -k 1 |
tee ${PRFX}-classes.dat |
awk -v n=${numcl} -v v=${vol} '
  BEGIN { i = 1 }
  { while ( i < $1 ) { print n; i++ } }
  { print $2 - 1; i++ }
  END { while ( i <= v ) { print n; i++ } }
' >${PRFX}-cls.dat

lost=$(
  awk -v n=${numcl} '
    BEGIN { m = 0 }
    $0 == n { m++ }
    END { print m }
  ' ${PRFX}-cls.dat
)

if [ ${lost} -gt 0 ]; then
  echo "! "
  echo "! NOTICE: ${lost} images lost in classification"
  echo "! "
fi

cp ${I3D} ${CLS}.i3d

cls=$( printf "%s-%03u" ${CLS} ${numcl} )

i3clsimport -cls ${numcl} ${CLS}.i3d ${msaset} ${cls} ${PRFX}-cls.dat
