#!/bin/bash
#
# i3selectcombine.sh  version 2.2  combine results of individual alignments
#

set -e

# set input file paths

dir="${SEL}"

export I3FILEPATH=".:${dir}"


# select

i3contents ${SEL}.i3d 2>&1 |
awk '
  /ref-.+-box-/ {
    s = $2
    sub( "ref-", "", $2 )
    sub( "-box-", " ", $2 )
    print s, $2
  }
' |
while read s r b; do
  echo -n "${r} ${b} "
  head -n 1 ${dir}/${s}.trf
done |
cat >${dir}/ref-ali.tmp

dim=$( awk '{ if ( NF > 12 ) { print 3 } else { print 2 }; exit }' ${dir}/ref-ali.tmp )

if [ ${dim} = 2 ]; then
  ori="0 0"
else
  ori="0 0 0"
fi

awk '
  NF > 12 { print; next }
  { print $1, $2, $3, $4, $5, 0, $6, $7, 0, $8, $9, 0, $10, $11, 0, 0, 0, 0, $12 } 
' ${dir}/ref-ali.tmp |
awk '
  BEGIN { n = 0 }
  {
    if ( f[$1] == 1 ) {
      i = x[$1]
    } else {
      i = n
      x[$1] = n++
      f[$1] = 1
    }
    if ( f[$2] == 1 ) {
      j = x[$2]
    } else {
      j = n
      x[$2] = n++
      f[$2] = 1
    }
    print i,j,$0
  }
' |
sort -n -k 21 |
awk '
  BEGIN { k = -1; m = -1; n = -1; p = 0 }
  $1 > m { m = $1 }
  $2 > n { n = $2 }
  p { c[$1,$2] = $21; c[$2,$1] = $21 }
  { k++; r[k] = $1; b[k] = $2; t[k] = $0 }
  END {
    if ( p ) print m "x" n
    if ( m > n ) { n = m }
    for ( i = 0; i <= n; i++ ) {
      f[i] = 0
    }
    if ( p ) {
      for ( i = 0; i <= n; i++ ) {
        c[i,i] = 1
      }
      for ( i = 0; i <= n; i++ ) {
        printf( "%3u  ", i )
        for ( j = 0; j <= n; j++ ) {
          printf( "%8.4f", c[i,j] )
        }
        printf( "\n" )
      }
    }
    f[r[k]] = 1
    f[b[k]] = 1
    if (  p ) print r[k], b[k], c[r[k],b[k]]
    if ( !p ) {
      print n + 1, r[k]
      print t[k]
    }
    while ( --k >= 0 ) {
      for ( i = k; i >= 0; i-- ) {
        if ( f[r[i]] ) {
          if ( f[b[i]] ) {
            break
          } else {
            f[b[i]] = 1
            if (  p ) print r[i], b[i], c[r[i],b[i]]
            if ( !p ) print t[i]
            break
          }
        } else {
          if ( f[b[i]] ) {
            f[r[i]] = 1
            if (  p ) print r[i], b[i], c[r[i],b[i]]
            if ( !p ) print t[i]
            break
          } else {
            continue
          }
        }
      }
      if ( i < 0 ) {
        if ( p ) print "? ?"
        break
      } else {
        while ( i < k ) {
          r[i] = r[i+1]
          b[i] = b[i+1]
          t[i] = t[i+1]
          i++
        }
      }
    }
    for ( i = 0; i <= n; i++ ) {
      if ( f[i] == 0 ) {
        print "ERROR"; exit 1
      }
    }
  }
' |
awk '
  function trav( b, k ) {
    if ( b == a ) return 1
    if ( pp ) {
      printf( "j=%u: ", j )
      for ( k = 0; k < j; k++ ) {
        printf( " %u", tr[k] )
      }
      printf( "\n" )
    }
    for ( k = 0; k < j; k++ ) {
      if ( tr[k] == b ) return 0
    }
    for ( k = 0; k < nn[b]; k++ ) {
      if ( p ) {
        tp[j] = b"->"an[b,k]
        if ( pp ) printf( "j=%u:  try %s\n", j, tp[j] )
      }
      tr[j] = b
      tt[j] = "-1 " ant[b,k]
      j++
      s = trav( an[b,k] )
      if ( s ) return s
      j--
      if ( pp ) printf( "j=%u:  try %s fail\n", j, tp[j] )
    }
    for ( k = 0; k < np[b]; k++ ) {
      if ( p ) {
        tp[j] = ap[b,k]"<-"b
        if ( pp ) printf( "j=%u:  try %s\n", j, tp[j] )
      }
      tr[j] = b
      tt[j] = "+1 " apt[b,k]
      j++
      s = trav( ap[b,k] )
      if ( s ) return s
      j--
      if ( pp ) printf( "j=%u:  try %s fail\n", j, tp[j] )
    }
    return 0
  }
  function ptrav( b ) {
    if ( p ) {
      printf( "%u:", b )
      for ( k = 0; k < j; k++ ) {
        printf( " %s ", tp[k] )
      }
      printf( "\n" )
    }
    print "00", b, b, x[b], x[b], "***-***-*** 0 0 0  0 0 0  1 0 0  0 1 0  0 0 1"
    for ( k = 0; k < j; k++ ) {
      print tt[k]
    }
    print "99", b, b, x[b], x[b], "***-***-*** 0 0 0  0 0 0  1 0 0  0 1 0  0 0 1"
  }
  BEGIN { p = 0; pp = 0 }
  NR == 1 {
    n = $1; a = $2
    for ( i = 0; i < n; i++ ) {
      np[i] = 0; nn[i] = 0
    }
    if ( p ) print "ref",a
    next
  }
  p { print $1, $2 }
  {
    $21 = ""
    ant[$1,nn[$1]] = $0; an[$1,nn[$1]++] = $2
    apt[$2,np[$2]] = $0; ap[$2,np[$2]++] = $1
    x[$1] = $3; x[$2] = $4
  }
  END {
    if ( p ) {
      for ( i = 0; i < n; i++ ) {
        printf( "p %u:", i )
        for ( j = 0; j < np[i]; j++ ) {
          printf( " %u", ap[i,j] )
        }
        printf( "\n" )
      }
      for ( i = 0; i < n; i++ ) {
        printf( "n %u:", i )
        for ( j = 0; j < nn[i]; j++ ) {
          printf( " %u", an[i,j] )
        }
        printf( "\n" )
      }
    }
    for ( i = 0; i < n; i++ ) {
      if ( pp ) print "trav", i
      j = 0; s = trav( i )
      ptrav( i ) 
    }
  }
' |
while read m r b cr cb id x y z dx dy dz a11 a12 a13 a21 a22 a23 a31 a32 a33 dummy; do
  set -e
  if [ ${dim} = 2 ]; then
    trf="${dx} ${dy} ${a11} ${a12} ${a21} ${a22}"
  else
    trf="${dx} ${dy} ${dz} ${a11} ${a12} ${a13} ${a21} ${a22} ${a23} ${a31} ${a32} ${a33}"
  fi
  if [ "${m}" = "00" ]; then
    A=${trf}
  elif [ "${m}" = "+1" ]; then
    A=$( i3trf ${trf} mul ${A} )
  elif [ "${m}" = "-1" ]; then
    trf1=$( i3trf inv ${trf} )
    A=$( i3trf ${trf1} mul ${A} )
  elif [ "${m}" = "99" ]; then
    echo box-${cb} ${ori} ${A}
  fi
done  >${dir}/ref-ali.trf

rm -f ${dir}/ref-ali.tmp

awk '{ $1 = $1"-avg"; print }' ${dir}/ref-ali.trf >${dir}/ref-avg-ali.trf


i3d=${SEL}-tmp.i3d

cp ${SEL}.i3d ${i3d}

i3dataimport -dim ${DIM} ${i3d} refs-avg ${dir}/ref-avg-ali.trf

i3window -clp -s ${MOTIFSIZE} ${i3d} refs-avg ${SEL}.img

mv -f ${i3d} ${SEL}.i3d

# create montages

if [ x${SELMONT} = xtrue ]; then

  i3montage -col ${SELMONTCOL:-10} ${SEL}.img ${SEL}-mont.img

  if [ ${DIM} = 3 -a y${YPERM} = ytrue ]; then
    i3permute -perm 1 0 0 0 0 1 0 -1 0 ${SEL}.img ${SEL}-y.img
    i3montage -col ${SELMONTCOL:-10} ${SEL}-y.img ${SEL}-y-mont.img
    rm -f ${SEL}-y.img
  fi

fi
